// 09.2004 by Bernd Kirchhoff
// Opens a Form in an artful way
// Call Show() in the show-event of a form and see what happens
//
// Use the following code in the close-event of a form
// Close();


unit ArtForm;
interface



uses Windows,Classes,Graphics,Controls,Forms,ExtCtrls;

type

TArtForm = class(TComponent)
private
  timer:TTimer;
  parentHandle:HWND;
  parentForm:TForm;
  regionHandle:HRGN;
  offX,offY:double;
  anz:double;
  useTyp:Integer;
  parentFormWidth,parentFormHeight,parentFormWidthMiddle,parentFormHeightMiddle:Integer;
  FTyp:Integer;
  FSteps:Integer;
  flagClose:Boolean;
  flagIsClosed:Boolean;
  angle:Real;
  procedure timerFunction(owner:TObject);
  procedure SetTyp(value:Integer);
  procedure SetSteps(value:Integer);
  procedure SetTypValues;
  function IsClosed:Boolean;
protected
public
   constructor Create(AOwner: TComponent); override;
   destructor Destroy; override;
   procedure Show;overload;
   procedure Show(typ:Integer);overload;
   procedure ShowLock;
   procedure Close;overload;
   procedure Close(typ:Integer);overload;
published
   property Typ: integer  read FTyp write SetTyp default -1;
   property Steps: integer read FSteps write SetSteps default 100;
end;


const maxEffects=54;

implementation
{$R artform.Res}

//---------------------------------------------------------------------------

Procedure _RotateRgnEx(Var ARgn : HRGN; ADegree : Real);
Var
  wXFORM  : XFORM;    // transformation structure, see Windows API
  kRgnD   : DWord;    // count of RGNDATA structures in region
  RgnData : PRgnData; // pointer to region data
  Rt      : TRect;
  kX, kY  : Integer;
begin
  if (ARgn = 0) or (ADegree = 0) then
    Exit;

  // Get region's surrounding rectangular
  GetRgnBox(ARgn, Rt);

  // Move source region so that the centre of its surrounding rectangular
  // goes to the left top corner of a window
  kX := Rt.Left+(Rt.Right -Rt.Left) div 2;
  kY := Rt.Top +(Rt.Bottom-Rt.Top) div 2;
  OffsetRgn(ARgn, -kX, -kY);

  // Fill XFORM according to task (rotate region)
  FillChar(wXFORM, SizeOf(wXFORM), #0);
  wXFORM.eM11 := Cos(ADegree/180*pi);
  wXFORM.eM12 := -Sin(ADegree/180*pi);
  wXFORM.eM21 := -wXFORM.eM12;
  wXFORM.eM22 := wXFORM.eM11;

  // Prepare buffer to store region data
  kRgnD := GetRegionData(ARgn, 0, nil);
  GetMem(RgnData, SizeOf(RGNDATA) * kRgnD);
  // ..and fill the buffer with region's data
  GetRegionData(ARgn, kRgnD, RgnData);
  // ..delete source region
  DeleteObject(ARgn);

  // Create new region using data in the buffer and transformation wXFORM
  ARgn := ExtCreateRegion(@wXFORM, kRgnD, RgnData^);
  // Move output region to the origin place
  OffsetRgn(ARgn, kX, kY);
  FreeMem(RgnData);
end;


Function _RotateRgn(ARgn : HRGN; ADegree : Real) : HRGN;
Var
  wXFORM  : XFORM;    // transformation structure, see Windows API
  kRgnD   : DWord;    // count of RGNDATA structures in region
  RgnData : PRgnData; // pointer to region data
  Rt      : TRect;
  kX, kY  : Integer;
begin
  if (ARgn = 0) or (ADegree = 0) then
    Exit;

  // Get region's surrounding rectangular
  GetRgnBox(ARgn, Rt);

  // Move source region so that the centre of its surrounding rectangular
  // goes to the left top corner of a window
  kX := Rt.Left+(Rt.Right -Rt.Left) div 2;
  kY := Rt.Top +(Rt.Bottom-Rt.Top) div 2;
  OffsetRgn(ARgn, -kX, -kY);

  // Fill XFORM according to task (rotate region)
  FillChar(wXFORM, SizeOf(wXFORM), #0);
  wXFORM.eM11 := Cos(ADegree/180*pi);
  wXFORM.eM12 := -Sin(ADegree/180*pi);
  wXFORM.eM21 := -wXFORM.eM12;
  wXFORM.eM22 := wXFORM.eM11;

  // Prepare buffer to store region data
  kRgnD := GetRegionData(ARgn, 0, nil);
  GetMem(RgnData, SizeOf(RGNDATA) * kRgnD);
  // ..and fill the buffer with region's data
  GetRegionData(ARgn, kRgnD, RgnData);
  // ..move source region to its initial position
  OffsetRgn(ARgn, kX, kY);

  // Create output region using data in the buffer and transformation wXFORM
  Result := ExtCreateRegion(@wXFORM, kRgnD, RgnData^);
  // Move output region on a place of source region
  OffsetRgn(Result, kX, kY);
  FreeMem(RgnData);
end;


constructor TArtForm.Create(AOwner: TComponent);
begin
   inherited Create(AOwner);
   Randomize();
   regionHandle:=0;
   parentForm:=TForm(AOwner);
   parentHandle:=parentForm.Handle;

   timer:=TTimer.Create(self);
   timer.Interval:=10;
   timer.Enabled:=false;
   timer.OnTimer:=timerFunction;

   flagClose:=false;
   flagIsClosed:=false;
   anz:=100;
   angle:=0;
   FSteps:=100;
end;

//---------------------------------------------------------------------------

destructor TArtForm.Destroy;
begin
  if timer<>nil then
    timer.Free;
  if regionHandle<>0 then
    DeleteObject(regionHandle);
  inherited Destroy;
end;

//---------------------------------------------------------------------------

procedure TArtForm.Show(typ:Integer);
begin
  SetTyp(typ);
  Show;
end;

//---------------------------------------------------------------------------

procedure TArtForm.Show;
begin
   if parentForm.WindowState=wsMinimized then
     Exit;
   if timer.Enabled=true then
     Exit; // ist noch am laufen

   angle:=0;
   flagIsClosed:=false;
   parentFormWidth:=parentForm.Width;
   parentFormHeight:=parentForm.Height;
   parentFormWidthMiddle:=parentFormWidth div 2;
   parentFormHeightMiddle:=parentFormHeight div 2;

   SetTypValues;
   // Damit es zu Anfang nicht flackert
   if regionHandle<>0 then
     DeleteObject(regionHandle);

   if flagClose=false then
   begin
     regionHandle:=CreateRectRgn(0,0,1,1);
     SetWindowRgn(parentHandle,regionHandle,true);
     DeleteObject(regionHandle);
   end;
   regionHandle:=0;
   timer.Tag:=0;
   timer.Enabled:=true;
end;

//---------------------------------------------------------------------------

procedure TArtForm.ShowLock;
begin
  regionHandle:=CreateRectRgn ( 0,0,1,1);
  SetWindowRgn(parentHandle,regionHandle,true);
  DeleteObject(regionHandle);
end;

//---------------------------------------------------------------------------

procedure TArtForm.Close(typ:Integer);
begin
  SetTyp(typ);
  Close;
end;

//---------------------------------------------------------------------------

procedure TArtForm.Close;
begin
  flagClose:=true;
  Show;
  repeat until IsClosed=true;
end;

//---------------------------------------------------------------------------

function TArtForm.IsClosed:Boolean;
begin
  Result:=flagIsClosed;
  if flagIsClosed=false then
    Application.ProcessMessages;
end;

//---------------------------------------------------------------------------

procedure TArtForm.SetTyp(value:Integer);
begin
  if value<0 then
    value:=0;
  if value>maxEffects then
    value:=0;

  FTyp:=value;
end;

//---------------------------------------------------------------------------

procedure TArtForm.SetSteps(value:Integer);
begin
  if value<5 then
    value:=5;
  if value>200 then
    value:=200;

  FSteps:=value;
  anz:=FSteps;
end;

//---------------------------------------------------------------------------

procedure TArtForm.SetTypValues;
begin
  angle:=160.0/FSteps;
  useTyp:=FTyp;
  if FTyp=0 then
    useTyp:=random(maxEffects)+1;

  case useTyp of
    1,2,3,4,5,6,21,25,29,30,31,32,33,34,35,36,37,38,39,40:
    begin
      offX:=parentFormWidth/(anz*2);
      offY:=parentFormHeight/(anz*2);
    end;
    7,8,9,10,11,24,41,42,43,44,45,46,47,48,49,50:
    begin // Kreise
      offX:=(parentFormWidth+parentFormWidthMiddle)/(anz*2);
      offY:=(parentFormHeight+parentFormHeightMiddle)/(anz*2);
    end;
    12,13,14,15,16,17,51,52,53,54:
    begin
      offX:=parentFormWidth/anz*2;
      offY:=parentFormHeight/anz*2;
      angle:=360.0/FSteps;
    end;
    18,19,20,28:
    begin
      offX:=parentFormWidth/anz;
      offY:=parentFormHeight/anz;
    end;
    22,23:
    begin
      offX:=parentFormWidth/(anz*2)*2;
      offY:=parentFormHeight/anz*2;
    end;
    26,27:
    begin
      offX:=parentFormWidth/anz*2;
      offY:=parentFormHeight/(anz*2)*2;
    end;
  end;
end;

//---------------------------------------------------------------------------

procedure TArtForm.timerFunction(owner:TObject);
var r1:HRGN;
    xOff,yOff:Integer;
    points:array[0..7] of TPoint;
begin
  timer.Enabled:=false;
  if (regionHandle<>0) then
    DeleteObject(regionHandle);

  timer.Tag:=timer.Tag+1;
  if flagClose=true then
    timer.Tag:=timer.Tag+1; // Beim Schlieen doppel da das invertieren so lange dauert
  r1:=0;
  xOff:=Round(offX*timer.Tag);
  yOff:=Round(offY*timer.Tag);

  case useTyp of
    1,29,30:
    begin
      // Von links und rechts nach innen
      r1:=CreateRectRgn(parentFormWidth-xOff,0,parentFormWidth,parentFormHeight);
      regionHandle:=CreateRectRgn( 0,0,xOff,parentFormHeight);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    2,31,32:
    begin
      // Von oben und unten nach innen
      r1:=CreateRectRgn(0,parentFormHeight-yOff,parentFormWidth,parentFormHeight);
      regionHandle:=CreateRectRgn( 0,0,parentFormWidth,yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    3,33,34:
    begin
      // Von links und rechts nach innen und von oben und unten nach innen
      regionHandle:=CreateRectRgn( 0,0,parentFormWidth,parentFormHeight);
      r1:=CreateRectRgn(xOff,yOff,parentFormWidth-xOff,parentFormHeight-yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_XOR);
    end;
    4,35,36:
    begin
      // Von der Mitte nach auen wachsendes Quadrat
      regionHandle:=CreateRectRgn(parentFormWidthMiddle-xOff,
                                  parentFormHeightMiddle-yOff,
                                  parentFormWidthMiddle+xOff,
                                  parentFormHeightMiddle+yOff);
    end;
    5,37,38:
    begin
      // Mittlerer Streifen nach links und rechts auen
      regionHandle:=CreateRectRgn(parentFormWidthMiddle-xOff,
                                  0,
                                  parentFormWidthMiddle+xOff,
                                  parentFormHeight);
    end;
    6,39,40:
    begin
      // Mittlerer Streifen nach oben und unten auen
      regionHandle:=CreateRectRgn(0,
                                  parentFormHeightMiddle-yOff,
                                  parentFormWidth,
                                  parentFormHeightMiddle+yOff);
    end;
    7,41,42:
    begin
      // Von der Mitte nach auen wachsender Kreis
      regionHandle:=CreateEllipticRgn(parentFormWidthMiddle-xOff,
                                  parentFormHeightMiddle-yOff,
                                  parentFormWidthMiddle+xOff,
                                  parentFormHeightMiddle+yOff);
    end;
    8,43,44:
    begin
      // Von auen zur  Mitte wachsender Kreis
      regionHandle:=CreateRectRgn( 0,0,parentFormWidth,parentFormHeight);
      r1:=CreateEllipticRgn(xOff-Round(parentFormWidthMiddle/2),
                            yOff-Round(parentFormHeightMiddle/2),
                            parentFormWidth-xOff+Round(parentFormWidthMiddle/2),
                            parentFormHeight-yOff+Round(parentFormHeightMiddle/2));
      CombineRgn(regionHandle,regionHandle,r1,RGN_XOR);
    end;
    9,45,46:
    begin
      // Von links und rechts  zur  Mitte wachsende Halbkreise
      regionHandle:=CreateEllipticRgn(-xOff,parentFormHeightMiddle+yOff,xOff,parentFormHeightMiddle-yOff);
      r1:=CreateEllipticRgn(parentFormWidth-xOff,parentFormHeightMiddle+yOff,parentFormWidth+xOff,parentFormHeightMiddle-yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    10,47,48:
    begin
      // Von oben und unten  zur  Mitte wachsende Halbkreise
      regionHandle:=CreateEllipticRgn(parentFormWidthMiddle-xOff,yOff,parentFormWidthMiddle+xOff,-yOff);
      r1:=CreateEllipticRgn(parentFormWidthMiddle-xOff,parentFormHeight+yOff,parentFormWidthMiddle+xOff,parentFormHeight-yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    11,49,50:
    begin
      // Von links und rechts und unten und oben zur  Mitte wachsende Halbkreise
      regionHandle:=CreateEllipticRgn(-xOff,parentFormHeightMiddle+yOff,xOff,parentFormHeightMiddle-yOff);
      r1:=CreateEllipticRgn(parentFormWidth-xOff,parentFormHeightMiddle+yOff,parentFormWidth+xOff,parentFormHeightMiddle-yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      r1:=CreateEllipticRgn(parentFormWidthMiddle-xOff,yOff,parentFormWidthMiddle+xOff,-yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      r1:=CreateEllipticRgn(parentFormWidthMiddle-xOff,parentFormHeight+yOff,parentFormWidthMiddle+xOff,parentFormHeight-yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    12,51,52:
    begin
      // von den Ecken links oben und rechts unten Dreiecke zur Mitte
      points[0]:=Point(0,yOff);
      points[1]:=Point(0,0);
      points[2]:=Point(xOff,0);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(parentFormWidth-xOff,parentFormHeight);
      points[1]:=Point(parentFormWidth,parentFormHeight);
      points[2]:=Point(parentFormWidth,parentFormHeight-yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    13,53,54:
    begin
      // von den Ecken links unten und rechts oben Dreiecke zur Mitte
      points[0]:=Point(0,parentFormHeight-yOff);
      points[1]:=Point(0,parentFormHeight);
      points[2]:=Point(xOff,parentFormHeight);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(parentFormWidth-xOff,0);
      points[1]:=Point(parentFormWidth,0);
      points[2]:=Point(parentFormWidth,yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    14:
    begin
      // von den Ecken  Dreiecke zur Mitte
      points[0]:=Point(0,yOff);
      points[1]:=Point(0,0);
      points[2]:=Point(xOff,0);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(parentFormWidth-xOff,parentFormHeight);
      points[1]:=Point(parentFormWidth,parentFormHeight);
      points[2]:=Point(parentFormWidth,parentFormHeight-yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(0,parentFormHeight-yOff);
      points[1]:=Point(0,parentFormHeight);
      points[2]:=Point(xOff,parentFormHeight);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(parentFormWidth-xOff,0);
      points[1]:=Point(parentFormWidth,0);
      points[2]:=Point(parentFormWidth,yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    15:
    begin
      // Scherenschnitt von oben und unten
      points[0]:=Point(0,0);
      points[1]:=Point(parentFormWidth,0);
      points[2]:=Point(parentFormWidth,yOff);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(parentFormWidth,parentFormHeight);
      points[1]:=Point(0,parentFormHeight);
      points[2]:=Point(0,parentFormHeight-yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    16:
    begin
      // Scherenschnitt von links und rechts
      points[0]:=Point(parentFormWidth,0);
      points[1]:=Point(parentFormWidth,parentFormHeight);
      points[2]:=Point(parentFormWidth-xOff,parentFormHeight);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(0,parentFormHeight);
      points[1]:=Point(0,0);
      points[2]:=Point(xOff,0);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    17:
    begin
      // Scherenschnitt von oben und unten, links und rechts
      points[0]:=Point(0,0);
      points[1]:=Point(parentFormWidth,0);
      points[2]:=Point(parentFormWidth,yOff);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(parentFormWidth,parentFormHeight);
      points[1]:=Point(0,parentFormHeight);
      points[2]:=Point(0,parentFormHeight-yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(parentFormWidth,0);
      points[1]:=Point(parentFormWidth,parentFormHeight);
      points[2]:=Point(parentFormWidth-xOff,parentFormHeight);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(0,parentFormHeight);
      points[1]:=Point(0,0);
      points[2]:=Point(xOff,0);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    18:
    begin
      // Kreise die von den Mittelachsen heraus wachsen
      regionHandle:=CreateEllipticRgn(0,parentFormHeightMiddle-yOff,parentFormWidth,parentFormHeightMiddle+yOff);
      r1:=CreateEllipticRgn(parentFormWidthMiddle-xOff,0,parentFormWidthMiddle+xOff,parentFormHeight);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    19:
    begin
      // Dreiecke die von den Mitten der Seiten mit der Spitze nach innen wachsen
      points[0]:=Point(parentFormWidthMiddle-xOff,0);
      points[1]:=Point(parentFormWidthMiddle+xOff,0);
      points[2]:=Point(parentFormWidthMiddle,yOff);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(parentFormWidthMiddle-xOff,parentFormHeight);
      points[1]:=Point(parentFormWidthMiddle+xOff,parentFormHeight);
      points[2]:=Point(parentFormWidthMiddle,parentFormHeight-yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(0,parentFormHeightMiddle-yOff);
      points[1]:=Point(0,parentFormHeightMiddle+yOff);
      points[2]:=Point(xOff,parentFormHeightMiddle);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(parentFormWidth,parentFormHeightMiddle-yOff);
      points[1]:=Point(parentFormWidth,parentFormHeightMiddle+yOff);
      points[2]:=Point(parentFormWidth-xOff,parentFormHeightMiddle);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      if yOff>=parentFormHeightMiddle then
        timer.Tag:=1000;
    end;
    20:
    begin
      // Scherenschnitt von oben und unten, links und rechts
      points[0]:=Point(0,0);
      points[1]:=Point(xOff,yOff);
      points[2]:=Point(0,yOff);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(parentFormWidth,0);
      points[1]:=Point(parentFormWidth-xOff,0);
      points[2]:=Point(parentFormWidth-xOff,yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(parentFormWidth,parentFormHeight);
      points[1]:=Point(parentFormWidth-xOff,parentFormHeight-yOff);
      points[2]:=Point(parentFormWidth,parentFormHeight-yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(0,parentFormHeight);
      points[1]:=Point(xOff,parentFormHeight);
      points[2]:=Point(xOff,parentFormHeight-yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
    end;
    21:
    begin
      // 4 zackiger Stern
      points[0]:=Point(parentFormWidthMiddle,0);
      points[1]:=Point(parentFormWidthMiddle-xOff,parentFormHeightMiddle-yOff);
      points[2]:=Point(0,parentFormHeightMiddle);
      points[3]:=Point(parentFormWidthMiddle-xOff,parentFormHeightMiddle+yOff);
      points[4]:=Point(parentFormWidthMiddle,parentFormHeight);
      points[5]:=Point(parentFormWidthMiddle+xOff,parentFormHeightMiddle+yOff);
      points[6]:=Point(parentFormWidth,parentFormHeightMiddle);
      points[7]:=Point(parentFormWidthMiddle+xOff,parentFormHeightMiddle-yOff);
      regionHandle:=CreatePolygonRgn(points,8,WINDING);
    end;
    22,23:
    begin
      // 1 Dreieck kommt von oben mitte, zwei von unten zwei halbe von oben seite
      points[0]:=Point(parentFormWidthMiddle-(xOff div 2),0);
      points[1]:=Point(parentFormWidthMiddle,yOff);
      points[2]:=Point(parentFormWidthMiddle+(xOff div 2),0);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point((parentFormWidthMiddle-xOff) div 2,parentFormHeight);
      points[1]:=Point((parentFormWidthMiddle) div 2,parentFormHeight-yOff);
      points[2]:=Point((parentFormWidthMiddle+xOff) div 2,parentFormHeight);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point((parentFormWidthMiddle-xOff+parentFormWidth) div 2,parentFormHeight);
      points[1]:=Point((parentFormWidthMiddle+parentFormWidth) div 2,parentFormHeight-yOff);
      points[2]:=Point((parentFormWidthMiddle+xOff+parentFormWidth) div 2,parentFormHeight);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(0,0);
      points[1]:=Point(0,yOff);
      points[2]:=Point(xOff div 2,0);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(parentFormWidth,0);
      points[1]:=Point(parentFormWidth,yOff);
      points[2]:=Point(parentFormWidth-(xOff div 2),0);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      if useTyp=23 then
        _RotateRgnEx(regionHandle,180);
    end;
    24:
    begin
      // Von der Mitte nach auen und von auen zur Mitte wachsender Kreis
      regionHandle:=CreateRectRgn( 0,0,parentFormWidth,parentFormHeight);
      r1:=CreateEllipticRgn(xOff-Round(parentFormWidthMiddle/2),
                            yOff-Round(parentFormHeightMiddle/2),
                            parentFormWidth-xOff+Round(parentFormWidthMiddle/2),
                            parentFormHeight-yOff+Round(parentFormHeightMiddle/2));
      CombineRgn(regionHandle,regionHandle,r1,RGN_XOR);
      DeleteObject(r1);
      r1:=CreateEllipticRgn(parentFormWidthMiddle-xOff,
                                  parentFormHeightMiddle-yOff,
                                  parentFormWidthMiddle+xOff,
                                  parentFormHeightMiddle+yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      if yOff>=parentFormHeightMiddle then
        timer.Tag:=1000;
    end;
    25:
    begin
      // Von links und rechts nach innen und von oben und unten nach innen
      // Von der Mitte nach auen wachsendes Quadrat
      regionHandle:=CreateRectRgn( 0,0,parentFormWidth,parentFormHeight);
      r1:=CreateRectRgn(xOff,yOff,parentFormWidth-xOff,parentFormHeight-yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_XOR);
      DeleteObject(r1);
      r1:=CreateRectRgn(parentFormWidthMiddle-xOff,
                                  parentFormHeightMiddle-yOff,
                                  parentFormWidthMiddle+xOff,
                                  parentFormHeightMiddle+yOff);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      if yOff>=parentFormHeightMiddle/2 then
        timer.Tag:=1000;
    end;
    26,27:
    begin
      // 1 Dreieck kommt von links mitte, zwei von rechts zwei halbe von rechts seite
      points[0]:=Point(0,parentFormHeightMiddle-(yOff div 2));
      points[1]:=Point(xOff,parentFormHeightMiddle);
      points[2]:=Point(0,parentFormHeightMiddle+(yOff div 2));
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(parentFormWidth,(parentFormHeightMiddle-yOff) div 2);

      points[1]:=Point(parentFormWidth-xOff,parentFormHeightMiddle div 2);
      points[2]:=Point(parentFormWidth,(parentFormHeightMiddle+yOff) div 2);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(parentFormWidth,(parentFormHeightMiddle-yOff+parentFormHeight) div 2);
      points[1]:=Point(parentFormWidth-xOff,(parentFormHeightMiddle+parentFormHeight) div 2);
      points[2]:=Point(parentFormWidth,(parentFormHeightMiddle+yOff+parentFormHeight) div 2);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(0,0);
      points[1]:=Point(xOff,0);
      points[2]:=Point(0,yOff div 2);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(0,parentFormHeight);
      points[1]:=Point(xOff,parentFormHeight);
      points[2]:=Point(0,parentFormHeight-(yOff div 2));
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      if useTyp=26 then
        _RotateRgnEx(regionHandle,180);
    end;
    28:
    begin
      // 1 Dreiecke die nach auen und 2 nach innen wachsen
      points[0]:=Point(parentFormWidthMiddle,parentFormHeightMiddle);
      points[1]:=Point(parentFormWidthMiddle+xOff,parentFormHeightMiddle-yOff);
      points[2]:=Point(parentFormWidthMiddle-xOff,parentFormHeightMiddle-yOff);
      regionHandle:=CreatePolygonRgn(points,3,WINDING);
      points[0]:=Point(parentFormWidthMiddle,parentFormHeightMiddle);
      points[1]:=Point(parentFormWidthMiddle+xOff,parentFormHeightMiddle+yOff);
      points[2]:=Point(parentFormWidthMiddle-xOff,parentFormHeightMiddle+yOff);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(0,parentFormHeightMiddle-yOff);
      points[1]:=Point(0,parentFormHeightMiddle+yOff);
      points[2]:=Point(xOff,parentFormHeightMiddle);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      DeleteObject(r1);
      points[0]:=Point(parentFormWidth,parentFormHeightMiddle-yOff);
      points[1]:=Point(parentFormWidth,parentFormHeightMiddle+yOff);
      points[2]:=Point(parentFormWidth-xOff,parentFormHeightMiddle);
      r1:=CreatePolygonRgn(points,3,WINDING);
      CombineRgn(regionHandle,regionHandle,r1,RGN_OR);
      if yOff>=parentFormHeight/2 then
        timer.Tag:=1000;
    end;
  end;
  case useTyp of
     29,31,33,35,37,39,41,43,45,47,49,51,53:
      _RotateRgnEx(regionHandle,angle*timer.Tag);
     30,32,34,36,38,40,42,44,46,48,50,52,54:
      _RotateRgnEx(regionHandle,-angle*timer.Tag);
  end;

  if yOff>=parentFormHeight then
    timer.Tag:=1000;

  if flagClose=true then
  begin
    if r1<>0 then
      DeleteObject(r1);
    r1:=CreateRectRgn( 0,0,parentFormWidth,parentFormHeight);
    CombineRgn(regionHandle,regionHandle,r1,RGN_XOR);
  end;

  SetWindowRgn(parentHandle,regionHandle,true);

  if timer.Tag>=anz then
  begin
    if flagClose=false then
    begin
      SetWindowRgn(parentHandle,0,true); // Alles Anzeigen
    end else
      flagIsClosed:=true;
    flagClose:=false;
  end else
  begin
    timer.Enabled:=true;
  end;

  if r1<>0 then
    DeleteObject(r1);

  DeleteObject(regionHandle);
  regionHandle:=0;
end;

//---------------------------------------------------------------------------

end.
